@@ -68,9 +68,6 @@ main (int   argc,
 
     g_set_prgname ("nautilus");
 
-#ifdef HAVE_EXEMPI
-    xmp_init ();
-#endif
     nautilus_register_resource ();
     /* Run the nautilus application. */
     application = nautilus_application_new ();
